# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from ..types.application_type import ApplicationType
import typing


class DeviceComponentDetailsApplicationDetailsParams(typing_extensions.TypedDict):
    application_type: typing_extensions.NotRequired[ApplicationType]
    """
    The type of application.
    See [ApplicationType](#type-applicationtype) for possible values
    """

    version: typing_extensions.NotRequired[str]
    """
    The version of the application.
    """

    session_location: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The location_id of the session for the application.
    """

    device_code_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The id of the device code that was used to log in to the device.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing


class CatalogQueryItemsForTaxParams(typing_extensions.TypedDict):
    """
    The query filter to return the items containing the specified tax IDs.
    """

    tax_ids: typing.Sequence[str]
    """
    A set of `CatalogTax` IDs to be used to find associated `CatalogItem`s.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class CustomField(UncheckedBaseModel):
    """
    Describes a custom form field to add to the checkout page to collect more information from buyers during checkout.
    For more information,
    see [Specify checkout options](https://developer.squareup.com/docs/checkout-api/optional-checkout-configurations#specify-checkout-options-1).
    """

    title: str = pydantic.Field()
    """
    The title of the custom field.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing


class CashDrawerDeviceParams(typing_extensions.TypedDict):
    id: typing_extensions.NotRequired[str]
    """
    The device Square-issued ID
    """

    name: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The device merchant-specified name.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .time_range import TimeRange
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class SearchOrdersDateTimeFilter(UncheckedBaseModel):
    """
    Filter for `Order` objects based on whether their `CREATED_AT`,
    `CLOSED_AT`, or `UPDATED_AT` timestamps fall within a specified time range.
    You can specify the time range and which timestamp to filter for. You can filter
    for only one time range at a time.

    For each time range, the start time and end time are inclusive. If the end time
    is absent, it defaults to the time of the first request for the cursor.

    __Important:__ If you use the `DateTimeFilter` in a `SearchOrders` query,
    you must set the `sort_field` in [OrdersSort](entity:SearchOrdersSort)
    to the same field you filter for. For example, if you set the `CLOSED_AT` field
    in `DateTimeFilter`, you must set the `sort_field` in `SearchOrdersSort` to
    `CLOSED_AT`. Otherwise, `SearchOrders` throws an error.
    [Learn more about filtering orders by time range.](https://developer.squareup.com/docs/orders-api/manage-orders/search-orders#important-note-about-filtering-orders-by-time-range)
    """

    created_at: typing.Optional[TimeRange] = pydantic.Field(default=None)
    """
    The time range for filtering on the `created_at` timestamp. If you use this
    value, you must set the `sort_field` in the `OrdersSearchSort` object to
    `CREATED_AT`.
    """

    updated_at: typing.Optional[TimeRange] = pydantic.Field(default=None)
    """
    The time range for filtering on the `updated_at` timestamp. If you use this
    value, you must set the `sort_field` in the `OrdersSearchSort` object to
    `UPDATED_AT`.
    """

    closed_at: typing.Optional[TimeRange] = pydantic.Field(default=None)
    """
    The time range for filtering on the `closed_at` timestamp. If you use this
    value, you must set the `sort_field` in the `OrdersSearchSort` object to
    `CLOSED_AT`.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from ...core.client_wrapper import SyncClientWrapper
from ...requests.custom_attribute_definition import CustomAttributeDefinitionParams
from ...core.request_options import RequestOptions
from ...core.http_response import HttpResponse
from ...types.create_location_custom_attribute_definition_response import (
    CreateLocationCustomAttributeDefinitionResponse,
)
from ...core.serialization import convert_and_respect_annotation_metadata
from ...core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ...core.api_error import ApiError
from ...types.retrieve_location_custom_attribute_definition_response import (
    RetrieveLocationCustomAttributeDefinitionResponse,
)
from ...core.jsonable_encoder import jsonable_encoder
from ...types.update_location_custom_attribute_definition_response import (
    UpdateLocationCustomAttributeDefinitionResponse,
)
from ...types.delete_location_custom_attribute_definition_response import (
    DeleteLocationCustomAttributeDefinitionResponse,
)
from ...core.client_wrapper import AsyncClientWrapper
from ...core.http_response import AsyncHttpResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawCustomAttributeDefinitionsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create(
        self,
        *,
        custom_attribute_definition: CustomAttributeDefinitionParams,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CreateLocationCustomAttributeDefinitionResponse]:
        """
        Creates a location-related [custom attribute definition](entity:CustomAttributeDefinition) for a Square seller account.
        Use this endpoint to define a custom attribute that can be associated with locations.
        A custom attribute definition specifies the `key`, `visibility`, `schema`, and other properties
        for a custom attribute. After the definition is created, you can call
        [UpsertLocationCustomAttribute](api-endpoint:LocationCustomAttributes-UpsertLocationCustomAttribute) or
        [BulkUpsertLocationCustomAttributes](api-endpoint:LocationCustomAttributes-BulkUpsertLocationCustomAttributes)
        to set the custom attribute for locations.

        Parameters
        ----------
        custom_attribute_definition : CustomAttributeDefinitionParams
            The custom attribute definition to create. Note the following:
            - With the exception of the `Selection` data type, the `schema` is specified as a simple URL to the JSON schema
            definition hosted on the Square CDN. For more information, including supported values and constraints, see
            [Supported data types](https://developer.squareup.com/docs/devtools/customattributes/overview#supported-data-types).
            - `name` is required unless `visibility` is set to `VISIBILITY_HIDDEN`.

        idempotency_key : typing.Optional[str]
            A unique identifier for this request, used to ensure idempotency. For more information,
            see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CreateLocationCustomAttributeDefinitionResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/locations/custom-attribute-definitions",
            method="POST",
            json={
                "custom_attribute_definition": convert_and_respect_annotation_metadata(
                    object_=custom_attribute_definition, annotation=CustomAttributeDefinitionParams, direction="write"
                ),
                "idempotency_key": idempotency_key,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateLocationCustomAttributeDefinitionResponse,
                    construct_type(
                        type_=CreateLocationCustomAttributeDefinitionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(
        self, key: str, *, version: typing.Optional[int] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[RetrieveLocationCustomAttributeDefinitionResponse]:
        """
        Retrieves a location-related [custom attribute definition](entity:CustomAttributeDefinition) from a Square seller account.
        To retrieve a custom attribute definition created by another application, the `visibility`
        setting must be `VISIBILITY_READ_ONLY` or `VISIBILITY_READ_WRITE_VALUES`.

        Parameters
        ----------
        key : str
            The key of the custom attribute definition to retrieve. If the requesting application
            is not the definition owner, you must use the qualified key.

        version : typing.Optional[int]
            The current version of the custom attribute definition, which is used for strongly consistent
            reads to guarantee that you receive the most up-to-date data. When included in the request,
            Square returns the specified version or a higher version if one exists. If the specified version
            is higher than the current version, Square returns a `BAD_REQUEST` error.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[RetrieveLocationCustomAttributeDefinitionResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/locations/custom-attribute-definitions/{jsonable_encoder(key)}",
            method="GET",
            params={
                "version": version,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    RetrieveLocationCustomAttributeDefinitionResponse,
                    construct_type(
                        type_=RetrieveLocationCustomAttributeDefinitionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update(
        self,
        key: str,
        *,
        custom_attribute_definition: CustomAttributeDefinitionParams,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[UpdateLocationCustomAttributeDefinitionResponse]:
        """
        Updates a location-related [custom attribute definition](entity:CustomAttributeDefinition) for a Square seller account.
        Use this endpoint to update the following fields: `name`, `description`, `visibility`, or the
        `schema` for a `Selection` data type.
        Only the definition owner can update a custom attribute definition.

        Parameters
        ----------
        key : str
            The key of the custom attribute definition to update.

        custom_attribute_definition : CustomAttributeDefinitionParams
            The custom attribute definition that contains the fields to update. This endpoint
            supports sparse updates, so only new or changed fields need to be included in the request.
            Only the following fields can be updated:
            - `name`
            - `description`
            - `visibility`
            - `schema` for a `Selection` data type. Only changes to the named options or the maximum number of allowed
            selections are supported.

            For more information, see
            [Update a location custom attribute definition](https://developer.squareup.com/docs/location-custom-attributes-api/custom-attribute-definitions#update-custom-attribute-definition).
            To enable [optimistic concurrency](https://developer.squareup.com/docs/build-basics/common-api-patterns/optimistic-concurrency)
            control, specify the current version of the custom attribute definition.
            If this is not important for your application, `version` can be set to -1.

        idempotency_key : typing.Optional[str]
            A unique identifier for this request, used to ensure idempotency. For more information,
            see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[UpdateLocationCustomAttributeDefinitionResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/locations/custom-attribute-definitions/{jsonable_encoder(key)}",
            method="PUT",
            json={
                "custom_attribute_definition": convert_and_respect_annotation_metadata(
                    object_=custom_attribute_definition, annotation=CustomAttributeDefinitionParams, direction="write"
                ),
                "idempotency_key": idempotency_key,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateLocationCustomAttributeDefinitionResponse,
                    construct_type(
                        type_=UpdateLocationCustomAttributeDefinitionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete(
        self, key: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[DeleteLocationCustomAttributeDefinitionResponse]:
        """
        Deletes a location-related [custom attribute definition](entity:CustomAttributeDefinition) from a Square seller account.
        Deleting a custom attribute definition also deletes the corresponding custom attribute from
        all locations.
        Only the definition owner can delete a custom attribute definition.

        Parameters
        ----------
        key : str
            The key of the custom attribute definition to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[DeleteLocationCustomAttributeDefinitionResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/locations/custom-attribute-definitions/{jsonable_encoder(key)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DeleteLocationCustomAttributeDefinitionResponse,
                    construct_type(
                        type_=DeleteLocationCustomAttributeDefinitionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncRawCustomAttributeDefinitionsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create(
        self,
        *,
        custom_attribute_definition: CustomAttributeDefinitionParams,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CreateLocationCustomAttributeDefinitionResponse]:
        """
        Creates a location-related [custom attribute definition](entity:CustomAttributeDefinition) for a Square seller account.
        Use this endpoint to define a custom attribute that can be associated with locations.
        A custom attribute definition specifies the `key`, `visibility`, `schema`, and other properties
        for a custom attribute. After the definition is created, you can call
        [UpsertLocationCustomAttribute](api-endpoint:LocationCustomAttributes-UpsertLocationCustomAttribute) or
        [BulkUpsertLocationCustomAttributes](api-endpoint:LocationCustomAttributes-BulkUpsertLocationCustomAttributes)
        to set the custom attribute for locations.

        Parameters
        ----------
        custom_attribute_definition : CustomAttributeDefinitionParams
            The custom attribute definition to create. Note the following:
            - With the exception of the `Selection` data type, the `schema` is specified as a simple URL to the JSON schema
            definition hosted on the Square CDN. For more information, including supported values and constraints, see
            [Supported data types](https://developer.squareup.com/docs/devtools/customattributes/overview#supported-data-types).
            - `name` is required unless `visibility` is set to `VISIBILITY_HIDDEN`.

        idempotency_key : typing.Optional[str]
            A unique identifier for this request, used to ensure idempotency. For more information,
            see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CreateLocationCustomAttributeDefinitionResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/locations/custom-attribute-definitions",
            method="POST",
            json={
                "custom_attribute_definition": convert_and_respect_annotation_metadata(
                    object_=custom_attribute_definition, annotation=CustomAttributeDefinitionParams, direction="write"
                ),
                "idempotency_key": idempotency_key,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateLocationCustomAttributeDefinitionResponse,
                    construct_type(
                        type_=CreateLocationCustomAttributeDefinitionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(
        self, key: str, *, version: typing.Optional[int] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[RetrieveLocationCustomAttributeDefinitionResponse]:
        """
        Retrieves a location-related [custom attribute definition](entity:CustomAttributeDefinition) from a Square seller account.
        To retrieve a custom attribute definition created by another application, the `visibility`
        setting must be `VISIBILITY_READ_ONLY` or `VISIBILITY_READ_WRITE_VALUES`.

        Parameters
        ----------
        key : str
            The key of the custom attribute definition to retrieve. If the requesting application
            is not the definition owner, you must use the qualified key.

        version : typing.Optional[int]
            The current version of the custom attribute definition, which is used for strongly consistent
            reads to guarantee that you receive the most up-to-date data. When included in the request,
            Square returns the specified version or a higher version if one exists. If the specified version
            is higher than the current version, Square returns a `BAD_REQUEST` error.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[RetrieveLocationCustomAttributeDefinitionResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/locations/custom-attribute-definitions/{jsonable_encoder(key)}",
            method="GET",
            params={
                "version": version,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    RetrieveLocationCustomAttributeDefinitionResponse,
                    construct_type(
                        type_=RetrieveLocationCustomAttributeDefinitionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update(
        self,
        key: str,
        *,
        custom_attribute_definition: CustomAttributeDefinitionParams,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[UpdateLocationCustomAttributeDefinitionResponse]:
        """
        Updates a location-related [custom attribute definition](entity:CustomAttributeDefinition) for a Square seller account.
        Use this endpoint to update the following fields: `name`, `description`, `visibility`, or the
        `schema` for a `Selection` data type.
        Only the definition owner can update a custom attribute definition.

        Parameters
        ----------
        key : str
            The key of the custom attribute definition to update.

        custom_attribute_definition : CustomAttributeDefinitionParams
            The custom attribute definition that contains the fields to update. This endpoint
            supports sparse updates, so only new or changed fields need to be included in the request.
            Only the following fields can be updated:
            - `name`
            - `description`
            - `visibility`
            - `schema` for a `Selection` data type. Only changes to the named options or the maximum number of allowed
            selections are supported.

            For more information, see
            [Update a location custom attribute definition](https://developer.squareup.com/docs/location-custom-attributes-api/custom-attribute-definitions#update-custom-attribute-definition).
            To enable [optimistic concurrency](https://developer.squareup.com/docs/build-basics/common-api-patterns/optimistic-concurrency)
            control, specify the current version of the custom attribute definition.
            If this is not important for your application, `version` can be set to -1.

        idempotency_key : typing.Optional[str]
            A unique identifier for this request, used to ensure idempotency. For more information,
            see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[UpdateLocationCustomAttributeDefinitionResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/locations/custom-attribute-definitions/{jsonable_encoder(key)}",
            method="PUT",
            json={
                "custom_attribute_definition": convert_and_respect_annotation_metadata(
                    object_=custom_attribute_definition, annotation=CustomAttributeDefinitionParams, direction="write"
                ),
                "idempotency_key": idempotency_key,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateLocationCustomAttributeDefinitionResponse,
                    construct_type(
                        type_=UpdateLocationCustomAttributeDefinitionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete(
        self, key: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[DeleteLocationCustomAttributeDefinitionResponse]:
        """
        Deletes a location-related [custom attribute definition](entity:CustomAttributeDefinition) from a Square seller account.
        Deleting a custom attribute definition also deletes the corresponding custom attribute from
        all locations.
        Only the definition owner can delete a custom attribute definition.

        Parameters
        ----------
        key : str
            The key of the custom attribute definition to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[DeleteLocationCustomAttributeDefinitionResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/locations/custom-attribute-definitions/{jsonable_encoder(key)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DeleteLocationCustomAttributeDefinitionResponse,
                    construct_type(
                        type_=DeleteLocationCustomAttributeDefinitionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing


class CatalogItemOptionValueForItemVariationParams(typing_extensions.TypedDict):
    """
    A `CatalogItemOptionValue` links an item variation to an item option as
    an item option value. For example, a t-shirt item may offer a color option and
    a size option. An item option value would represent each variation of t-shirt:
    For example, "Color:Red, Size:Small" or "Color:Blue, Size:Medium".
    """

    item_option_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The unique id of an item option.
    """

    item_option_value_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The unique id of the selected value for the item option.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .location import LocationParams


class UpdateLocationResponseParams(typing_extensions.TypedDict):
    """
    The response object returned by the [UpdateLocation](api-endpoint:Locations-UpdateLocation) endpoint.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Information about errors encountered during the request.
    """

    location: typing_extensions.NotRequired[LocationParams]
    """
    The updated `Location` object.
    """



# This file was auto-generated by Fern from our API Definition.

from . import custom_attribute_definitions, custom_attributes, location_profiles, team_member_profiles

__all__ = ["custom_attribute_definitions", "custom_attributes", "location_profiles", "team_member_profiles"]



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .inventory_physical_count import InventoryPhysicalCountParams


class GetInventoryPhysicalCountResponseParams(typing_extensions.TypedDict):
    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """

    count: typing_extensions.NotRequired[InventoryPhysicalCountParams]
    """
    The requested [InventoryPhysicalCount](entity:InventoryPhysicalCount).
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from .team_member import TeamMemberParams
import typing
from .error import ErrorParams


class CreateTeamMemberResponseParams(typing_extensions.TypedDict):
    """
    Represents a response from a create request containing the created `TeamMember` object or error messages.
    """

    team_member: typing_extensions.NotRequired[TeamMemberParams]
    """
    The successfully created `TeamMember` object.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    The errors that occurred during the request.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing


class CatalogModifierOverrideParams(typing_extensions.TypedDict):
    """
    Options to control how to override the default behavior of the specified modifier.
    """

    modifier_id: str
    """
    The ID of the `CatalogModifier` whose default behavior is being overridden.
    """

    on_by_default: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    __Deprecated__: Use `on_by_default_override` instead.
    """

    hidden_online_override: typing_extensions.NotRequired[typing.Optional[typing.Any]]
    on_by_default_override: typing_extensions.NotRequired[typing.Optional[typing.Any]]



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from .money import MoneyParams
import typing


class GiftCardActivityLoadParams(typing_extensions.TypedDict):
    """
    Represents details about a `LOAD` [gift card activity type](entity:GiftCardActivityType).
    """

    amount_money: typing_extensions.NotRequired[MoneyParams]
    """
    The amount added to the gift card. This value is a positive integer.
    
    Applications that use a custom order processing system must specify this amount in the 
    [CreateGiftCardActivity](api-endpoint:GiftCardActivities-CreateGiftCardActivity) request.
    """

    order_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The ID of the [order](entity:Order) that contains the `GIFT_CARD` line item.
    
    Applications that use the Square Orders API to process orders must specify the order ID in the 
    [CreateGiftCardActivity](api-endpoint:GiftCardActivities-CreateGiftCardActivity) request.
    """

    line_item_uid: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The UID of the `GIFT_CARD` line item in the order that represents the additional funds for the gift card.
    
    Applications that use the Square Orders API to process orders must specify the line item UID
    in the [CreateGiftCardActivity](api-endpoint:GiftCardActivities-CreateGiftCardActivity) request.
    """

    reference_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    A client-specified ID that associates the gift card activity with an entity in another system. 
    
    Applications that use a custom order processing system can use this field to track information related to 
    an order or payment.
    """

    buyer_payment_instrument_ids: typing_extensions.NotRequired[typing.Optional[typing.Sequence[str]]]
    """
    The payment instrument IDs used to process the order for the additional funds, such as a credit card ID 
    or bank account ID. 
    
    Applications that use a custom order processing system must specify payment instrument IDs in 
    the [CreateGiftCardActivity](api-endpoint:GiftCardActivities-CreateGiftCardActivity) request.
    Square uses this information to perform compliance checks. 
    
    For applications that use the Square Orders API to process payments, Square has the necessary 
    instrument IDs to perform compliance checks.
    
    Each buyer payment instrument ID can contain a maximum of 255 characters.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
from .time_range import TimeRange
import pydantic
import typing
from .segment_filter import SegmentFilter
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class SearchAvailabilityFilter(UncheckedBaseModel):
    """
    A query filter to search for buyer-accessible availabilities by.
    """

    start_at_range: TimeRange = pydantic.Field()
    """
    The query expression to search for buy-accessible availabilities with their starting times falling within the specified time range.
    The time range must be at least 24 hours and at most 32 days long.
    For waitlist availabilities, the time range can be 0 or more up to 367 days long.
    """

    location_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The query expression to search for buyer-accessible availabilities with their location IDs matching the specified location ID.
    This query expression cannot be set if `booking_id` is set.
    """

    segment_filters: typing.Optional[typing.List[SegmentFilter]] = pydantic.Field(default=None)
    """
    The query expression to search for buyer-accessible availabilities matching the specified list of segment filters.
    If the size of the `segment_filters` list is `n`, the search returns availabilities with `n` segments per availability.
    
    This query expression cannot be set if `booking_id` is set.
    """

    booking_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The query expression to search for buyer-accessible availabilities for an existing booking by matching the specified `booking_id` value.
    This is commonly used to reschedule an appointment.
    If this expression is set, the `location_id` and `segment_filters` expressions cannot be set.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from .money import MoneyParams


class GiftCardActivityTransferBalanceFromParams(typing_extensions.TypedDict):
    """
    Represents details about a `TRANSFER_BALANCE_FROM` [gift card activity type](entity:GiftCardActivityType).
    """

    transfer_to_gift_card_id: str
    """
    The ID of the gift card to which the specified amount was transferred.
    """

    amount_money: MoneyParams
    """
    The amount deducted from the gift card for the transfer. This value is a positive integer.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .range import RangeParams


class CustomAttributeFilterParams(typing_extensions.TypedDict):
    """
    Supported custom attribute query expressions for calling the
    [SearchCatalogItems](api-endpoint:Catalog-SearchCatalogItems)
    endpoint to search for items or item variations.
    """

    custom_attribute_definition_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    A query expression to filter items or item variations by matching their custom attributes'
    `custom_attribute_definition_id` property value against the the specified id.
    Exactly one of `custom_attribute_definition_id` or `key` must be specified.
    """

    key: typing_extensions.NotRequired[typing.Optional[str]]
    """
    A query expression to filter items or item variations by matching their custom attributes'
    `key` property value against the specified key.
    Exactly one of `custom_attribute_definition_id` or `key` must be specified.
    """

    string_filter: typing_extensions.NotRequired[typing.Optional[str]]
    """
    A query expression to filter items or item variations by matching their custom attributes'
    `string_value`  property value against the specified text.
    Exactly one of `string_filter`, `number_filter`, `selection_uids_filter`, or `bool_filter` must be specified.
    """

    number_filter: typing_extensions.NotRequired[RangeParams]
    """
    A query expression to filter items or item variations with their custom attributes
    containing a number value within the specified range.
    Exactly one of `string_filter`, `number_filter`, `selection_uids_filter`, or `bool_filter` must be specified.
    """

    selection_uids_filter: typing_extensions.NotRequired[typing.Optional[typing.Sequence[str]]]
    """
    A query expression to filter items or item variations by matching  their custom attributes'
    `selection_uid_values` values against the specified selection uids.
    Exactly one of `string_filter`, `number_filter`, `selection_uids_filter`, or `bool_filter` must be specified.
    """

    bool_filter: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    A query expression to filter items or item variations by matching their custom attributes'
    `boolean_value` property values against the specified Boolean expression.
    Exactly one of `string_filter`, `number_filter`, `selection_uids_filter`, or `bool_filter` must be specified.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .money import MoneyParams
from ..types.tender_type import TenderType
from .tender_card_details import TenderCardDetailsParams
from .tender_cash_details import TenderCashDetailsParams
from .tender_bank_account_details import TenderBankAccountDetailsParams
from .tender_buy_now_pay_later_details import TenderBuyNowPayLaterDetailsParams
from .tender_square_account_details import TenderSquareAccountDetailsParams
from .additional_recipient import AdditionalRecipientParams


class TenderParams(typing_extensions.TypedDict):
    """
    Represents a tender (i.e., a method of payment) used in a Square transaction.
    """

    id: typing_extensions.NotRequired[str]
    """
    The tender's unique ID. It is the associated payment ID.
    """

    location_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The ID of the transaction's associated location.
    """

    transaction_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The ID of the tender's associated transaction.
    """

    created_at: typing_extensions.NotRequired[str]
    """
    The timestamp for when the tender was created, in RFC 3339 format.
    """

    note: typing_extensions.NotRequired[typing.Optional[str]]
    """
    An optional note associated with the tender at the time of payment.
    """

    amount_money: typing_extensions.NotRequired[MoneyParams]
    """
    The total amount of the tender, including `tip_money`. If the tender has a `payment_id`,
    the `total_money` of the corresponding [Payment](entity:Payment) will be equal to the
    `amount_money` of the tender.
    """

    tip_money: typing_extensions.NotRequired[MoneyParams]
    """
    The tip's amount of the tender.
    """

    processing_fee_money: typing_extensions.NotRequired[MoneyParams]
    """
    The amount of any Square processing fees applied to the tender.
    
    This field is not immediately populated when a new transaction is created.
    It is usually available after about ten seconds.
    """

    customer_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    If the tender is associated with a customer or represents a customer's card on file,
    this is the ID of the associated customer.
    """

    type: TenderType
    """
    The type of tender, such as `CARD` or `CASH`.
    See [TenderType](#type-tendertype) for possible values
    """

    card_details: typing_extensions.NotRequired[TenderCardDetailsParams]
    """
    The details of the card tender.
    
    This value is present only if the value of `type` is `CARD`.
    """

    cash_details: typing_extensions.NotRequired[TenderCashDetailsParams]
    """
    The details of the cash tender.
    
    This value is present only if the value of `type` is `CASH`.
    """

    bank_account_details: typing_extensions.NotRequired[TenderBankAccountDetailsParams]
    """
    The details of the bank account tender.
    
    This value is present only if the value of `type` is `BANK_ACCOUNT`.
    """

    buy_now_pay_later_details: typing_extensions.NotRequired[TenderBuyNowPayLaterDetailsParams]
    """
    The details of a Buy Now Pay Later tender.
    
    This value is present only if the value of `type` is `BUY_NOW_PAY_LATER`.
    """

    square_account_details: typing_extensions.NotRequired[TenderSquareAccountDetailsParams]
    """
    The details of a Square Account tender.
    
    This value is present only if the value of `type` is `SQUARE_ACCOUNT`.
    """

    additional_recipients: typing_extensions.NotRequired[typing.Optional[typing.Sequence[AdditionalRecipientParams]]]
    """
    Additional recipients (other than the merchant) receiving a portion of this tender.
    For example, fees assessed on the purchase by a third party integration.
    """

    payment_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The ID of the [Payment](entity:Payment) that corresponds to this tender.
    This value is only present for payments created with the v2 Payments API.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .terminal_refund import TerminalRefundParams


class GetTerminalRefundResponseParams(typing_extensions.TypedDict):
    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Information about errors encountered during the request.
    """

    refund: typing_extensions.NotRequired[TerminalRefundParams]
    """
    The requested `Refund`.
    """



# This file was auto-generated by Fern from our API Definition.

from ...core.client_wrapper import SyncClientWrapper
from .raw_client import RawShiftsClient
import typing
from ...types.sort_order import SortOrder
from ...core.request_options import RequestOptions
from ...core.pagination import SyncPager
from ...types.cash_drawer_shift_summary import CashDrawerShiftSummary
from ...types.list_cash_drawer_shifts_response import ListCashDrawerShiftsResponse
from ...core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ...core.api_error import ApiError
from ...types.get_cash_drawer_shift_response import GetCashDrawerShiftResponse
from ...types.cash_drawer_shift_event import CashDrawerShiftEvent
from ...core.jsonable_encoder import jsonable_encoder
from ...types.list_cash_drawer_shift_events_response import ListCashDrawerShiftEventsResponse
from ...core.client_wrapper import AsyncClientWrapper
from .raw_client import AsyncRawShiftsClient
from ...core.pagination import AsyncPager


class ShiftsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawShiftsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawShiftsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawShiftsClient
        """
        return self._raw_client

    def list(
        self,
        *,
        location_id: str,
        sort_order: typing.Optional[SortOrder] = None,
        begin_time: typing.Optional[str] = None,
        end_time: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        cursor: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[CashDrawerShiftSummary]:
        """
        Provides the details for all of the cash drawer shifts for a location
        in a date range.

        Parameters
        ----------
        location_id : str
            The ID of the location to query for a list of cash drawer shifts.

        sort_order : typing.Optional[SortOrder]
            The order in which cash drawer shifts are listed in the response,
            based on their opened_at field. Default value: ASC

        begin_time : typing.Optional[str]
            The inclusive start time of the query on opened_at, in ISO 8601 format.

        end_time : typing.Optional[str]
            The exclusive end date of the query on opened_at, in ISO 8601 format.

        limit : typing.Optional[int]
            Number of cash drawer shift events in a page of results (200 by
            default, 1000 max).

        cursor : typing.Optional[str]
            Opaque cursor for fetching the next page of results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[CashDrawerShiftSummary]
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        response = client.cash_drawers.shifts.list(
            location_id="location_id",
        )
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        _response = self._raw_client._client_wrapper.httpx_client.request(
            "v2/cash-drawers/shifts",
            method="GET",
            params={
                "location_id": location_id,
                "sort_order": sort_order,
                "begin_time": begin_time,
                "end_time": end_time,
                "limit": limit,
                "cursor": cursor,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListCashDrawerShiftsResponse,
                    construct_type(
                        type_=ListCashDrawerShiftsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list(
                    location_id=location_id,
                    sort_order=sort_order,
                    begin_time=begin_time,
                    end_time=end_time,
                    limit=limit,
                    cursor=_parsed_next,
                    request_options=request_options,
                )
                _items = _parsed_response.cash_drawer_shifts
                return SyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(
        self, shift_id: str, *, location_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> GetCashDrawerShiftResponse:
        """
        Provides the summary details for a single cash drawer shift. See
        [ListCashDrawerShiftEvents](api-endpoint:CashDrawers-ListCashDrawerShiftEvents) for a list of cash drawer shift events.

        Parameters
        ----------
        shift_id : str
            The shift ID.

        location_id : str
            The ID of the location to retrieve cash drawer shifts from.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetCashDrawerShiftResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.cash_drawers.shifts.get(
            shift_id="shift_id",
            location_id="location_id",
        )
        """
        response = self._raw_client.get(shift_id, location_id=location_id, request_options=request_options)
        return response.data

    def list_events(
        self,
        shift_id: str,
        *,
        location_id: str,
        limit: typing.Optional[int] = None,
        cursor: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[CashDrawerShiftEvent]:
        """
        Provides a paginated list of events for a single cash drawer shift.

        Parameters
        ----------
        shift_id : str
            The shift ID.

        location_id : str
            The ID of the location to list cash drawer shifts for.

        limit : typing.Optional[int]
            Number of resources to be returned in a page of results (200 by
            default, 1000 max).

        cursor : typing.Optional[str]
            Opaque cursor for fetching the next page of results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[CashDrawerShiftEvent]
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        response = client.cash_drawers.shifts.list_events(
            shift_id="shift_id",
            location_id="location_id",
        )
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        _response = self._raw_client._client_wrapper.httpx_client.request(
            f"v2/cash-drawers/shifts/{jsonable_encoder(shift_id)}/events",
            method="GET",
            params={
                "location_id": location_id,
                "limit": limit,
                "cursor": cursor,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListCashDrawerShiftEventsResponse,
                    construct_type(
                        type_=ListCashDrawerShiftEventsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list_events(
                    shift_id,
                    location_id=location_id,
                    limit=limit,
                    cursor=_parsed_next,
                    request_options=request_options,
                )
                _items = _parsed_response.cash_drawer_shift_events
                return SyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncShiftsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawShiftsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawShiftsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawShiftsClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        location_id: str,
        sort_order: typing.Optional[SortOrder] = None,
        begin_time: typing.Optional[str] = None,
        end_time: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        cursor: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[CashDrawerShiftSummary]:
        """
        Provides the details for all of the cash drawer shifts for a location
        in a date range.

        Parameters
        ----------
        location_id : str
            The ID of the location to query for a list of cash drawer shifts.

        sort_order : typing.Optional[SortOrder]
            The order in which cash drawer shifts are listed in the response,
            based on their opened_at field. Default value: ASC

        begin_time : typing.Optional[str]
            The inclusive start time of the query on opened_at, in ISO 8601 format.

        end_time : typing.Optional[str]
            The exclusive end date of the query on opened_at, in ISO 8601 format.

        limit : typing.Optional[int]
            Number of cash drawer shift events in a page of results (200 by
            default, 1000 max).

        cursor : typing.Optional[str]
            Opaque cursor for fetching the next page of results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[CashDrawerShiftSummary]
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.cash_drawers.shifts.list(
                location_id="location_id",
            )
            async for item in response:
                yield item
            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        _response = await self._raw_client._client_wrapper.httpx_client.request(
            "v2/cash-drawers/shifts",
            method="GET",
            params={
                "location_id": location_id,
                "sort_order": sort_order,
                "begin_time": begin_time,
                "end_time": end_time,
                "limit": limit,
                "cursor": cursor,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListCashDrawerShiftsResponse,
                    construct_type(
                        type_=ListCashDrawerShiftsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list(
                    location_id=location_id,
                    sort_order=sort_order,
                    begin_time=begin_time,
                    end_time=end_time,
                    limit=limit,
                    cursor=_parsed_next,
                    request_options=request_options,
                )
                _items = _parsed_response.cash_drawer_shifts
                return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(
        self, shift_id: str, *, location_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> GetCashDrawerShiftResponse:
        """
        Provides the summary details for a single cash drawer shift. See
        [ListCashDrawerShiftEvents](api-endpoint:CashDrawers-ListCashDrawerShiftEvents) for a list of cash drawer shift events.

        Parameters
        ----------
        shift_id : str
            The shift ID.

        location_id : str
            The ID of the location to retrieve cash drawer shifts from.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetCashDrawerShiftResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.cash_drawers.shifts.get(
                shift_id="shift_id",
                location_id="location_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.get(shift_id, location_id=location_id, request_options=request_options)
        return response.data

    async def list_events(
        self,
        shift_id: str,
        *,
        location_id: str,
        limit: typing.Optional[int] = None,
        cursor: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[CashDrawerShiftEvent]:
        """
        Provides a paginated list of events for a single cash drawer shift.

        Parameters
        ----------
        shift_id : str
            The shift ID.

        location_id : str
            The ID of the location to list cash drawer shifts for.

        limit : typing.Optional[int]
            Number of resources to be returned in a page of results (200 by
            default, 1000 max).

        cursor : typing.Optional[str]
            Opaque cursor for fetching the next page of results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[CashDrawerShiftEvent]
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.cash_drawers.shifts.list_events(
                shift_id="shift_id",
                location_id="location_id",
            )
            async for item in response:
                yield item
            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        _response = await self._raw_client._client_wrapper.httpx_client.request(
            f"v2/cash-drawers/shifts/{jsonable_encoder(shift_id)}/events",
            method="GET",
            params={
                "location_id": location_id,
                "limit": limit,
                "cursor": cursor,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListCashDrawerShiftEventsResponse,
                    construct_type(
                        type_=ListCashDrawerShiftEventsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list_events(
                    shift_id,
                    location_id=location_id,
                    limit=limit,
                    cursor=_parsed_next,
                    request_options=request_options,
                )
                _items = _parsed_response.cash_drawer_shift_events
                return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from ..requests.scheduled_shift import ScheduledShiftParams
from ..core.request_options import RequestOptions
from ..core.http_response import HttpResponse
from ..types.create_scheduled_shift_response import CreateScheduledShiftResponse
from ..core.serialization import convert_and_respect_annotation_metadata
from ..core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..requests.bulk_publish_scheduled_shifts_data import BulkPublishScheduledShiftsDataParams
from ..types.scheduled_shift_notification_audience import ScheduledShiftNotificationAudience
from ..types.bulk_publish_scheduled_shifts_response import BulkPublishScheduledShiftsResponse
from ..requests.scheduled_shift_query import ScheduledShiftQueryParams
from ..types.search_scheduled_shifts_response import SearchScheduledShiftsResponse
from ..types.retrieve_scheduled_shift_response import RetrieveScheduledShiftResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..types.update_scheduled_shift_response import UpdateScheduledShiftResponse
from ..types.publish_scheduled_shift_response import PublishScheduledShiftResponse
from ..requests.timecard import TimecardParams
from ..types.create_timecard_response import CreateTimecardResponse
from ..requests.timecard_query import TimecardQueryParams
from ..types.search_timecards_response import SearchTimecardsResponse
from ..types.retrieve_timecard_response import RetrieveTimecardResponse
from ..types.update_timecard_response import UpdateTimecardResponse
from ..types.delete_timecard_response import DeleteTimecardResponse
from ..core.client_wrapper import AsyncClientWrapper
from ..core.http_response import AsyncHttpResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawLaborClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create_scheduled_shift(
        self,
        *,
        scheduled_shift: ScheduledShiftParams,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CreateScheduledShiftResponse]:
        """
        Creates a scheduled shift by providing draft shift details such as job ID,
        team member assignment, and start and end times.

        The following `draft_shift_details` fields are required:
        - `location_id`
        - `job_id`
        - `start_at`
        - `end_at`

        Parameters
        ----------
        scheduled_shift : ScheduledShiftParams
            The scheduled shift with `draft_shift_details`.
            If needed, call [ListLocations](api-endpoint:Locations-ListLocations) to get location IDs,
            [ListJobs](api-endpoint:Team-ListJobs) to get job IDs, and [SearchTeamMembers](api-endpoint:Team-SearchTeamMembers)
            to get team member IDs and current job assignments.

            The `start_at` and `end_at` timestamps must be provided in the time zone + offset of the
            shift location specified in `location_id`. Example for Pacific Standard Time: 2024-10-31T12:30:00-08:00

        idempotency_key : typing.Optional[str]
            A unique identifier for the `CreateScheduledShift` request, used to ensure the
            [idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency)
            of the operation.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CreateScheduledShiftResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/labor/scheduled-shifts",
            method="POST",
            json={
                "idempotency_key": idempotency_key,
                "scheduled_shift": convert_and_respect_annotation_metadata(
                    object_=scheduled_shift, annotation=ScheduledShiftParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateScheduledShiftResponse,
                    construct_type(
                        type_=CreateScheduledShiftResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def bulk_publish_scheduled_shifts(
        self,
        *,
        scheduled_shifts: typing.Dict[str, BulkPublishScheduledShiftsDataParams],
        scheduled_shift_notification_audience: typing.Optional[ScheduledShiftNotificationAudience] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[BulkPublishScheduledShiftsResponse]:
        """
        Publishes 1 - 100 scheduled shifts. This endpoint takes a map of individual publish
        requests and returns a map of responses. When a scheduled shift is published, Square keeps
        the `draft_shift_details` field as is and copies it to the `published_shift_details` field.

        The minimum `start_at` and maximum `end_at` timestamps of all shifts in a
        `BulkPublishScheduledShifts` request must fall within a two-week period.

        Parameters
        ----------
        scheduled_shifts : typing.Dict[str, BulkPublishScheduledShiftsDataParams]
            A map of 1 to 100 key-value pairs that represent individual publish requests.

            - Each key is the ID of a scheduled shift you want to publish.
            - Each value is a `BulkPublishScheduledShiftsData` object that contains the
            `version` field or is an empty object.

        scheduled_shift_notification_audience : typing.Optional[ScheduledShiftNotificationAudience]
            Indicates whether Square should send email notifications to team members and
            which team members should receive the notifications. This setting applies to all shifts
            specified in the bulk operation. The default value is `AFFECTED`.
            See [ScheduledShiftNotificationAudience](#type-scheduledshiftnotificationaudience) for possible values

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[BulkPublishScheduledShiftsResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/labor/scheduled-shifts/bulk-publish",
            method="POST",
            json={
                "scheduled_shifts": convert_and_respect_annotation_metadata(
                    object_=scheduled_shifts,
                    annotation=typing.Dict[str, BulkPublishScheduledShiftsDataParams],
                    direction="write",
                ),
                "scheduled_shift_notification_audience": scheduled_shift_notification_audience,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    BulkPublishScheduledShiftsResponse,
                    construct_type(
                        type_=BulkPublishScheduledShiftsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def search_scheduled_shifts(
        self,
        *,
        query: typing.Optional[ScheduledShiftQueryParams] = OMIT,
        limit: typing.Optional[int] = OMIT,
        cursor: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[SearchScheduledShiftsResponse]:
        """
        Returns a paginated list of scheduled shifts, with optional filter and sort settings.
        By default, results are sorted by `start_at` in ascending order.

        Parameters
        ----------
        query : typing.Optional[ScheduledShiftQueryParams]
            Query conditions used to filter and sort the results.

        limit : typing.Optional[int]
            The maximum number of results to return in a single response page. The default value is 50.

        cursor : typing.Optional[str]
            The pagination cursor returned by the previous call to this endpoint. Provide
            this cursor to retrieve the next page of results for your original request. For more
            information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[SearchScheduledShiftsResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/labor/scheduled-shifts/search",
            method="POST",
            json={
                "query": convert_and_respect_annotation_metadata(
                    object_=query, annotation=ScheduledShiftQueryParams, direction="write"
                ),
                "limit": limit,
                "cursor": cursor,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SearchScheduledShiftsResponse,
                    construct_type(
                        type_=SearchScheduledShiftsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def retrieve_scheduled_shift(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[RetrieveScheduledShiftResponse]:
        """
        Retrieves a scheduled shift by ID.

        Parameters
        ----------
        id : str
            The ID of the scheduled shift to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[RetrieveScheduledShiftResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/labor/scheduled-shifts/{jsonable_encoder(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    RetrieveScheduledShiftResponse,
                    construct_type(
                        type_=RetrieveScheduledShiftResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update_scheduled_shift(
        self, id: str, *, scheduled_shift: ScheduledShiftParams, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[UpdateScheduledShiftResponse]:
        """
        Updates the draft shift details for a scheduled shift. This endpoint supports
        sparse updates, so only new, changed, or removed fields are required in the request.
        You must publish the shift to make updates public.

        You can make the following updates to `draft_shift_details`:
        - Change the `location_id`, `job_id`, `start_at`, and `end_at` fields.
        - Add, change, or clear the `team_member_id` and `notes` fields. To clear these fields,
        set the value to null.
        - Change the `is_deleted` field. To delete a scheduled shift, set `is_deleted` to true
        and then publish the shift.

        Parameters
        ----------
        id : str
            The ID of the scheduled shift to update.

        scheduled_shift : ScheduledShiftParams
            The scheduled shift with any updates in the `draft_shift_details` field.
            If needed, call [ListLocations](api-endpoint:Locations-ListLocations) to get location IDs,
            [ListJobs](api-endpoint:Team-ListJobs) to get job IDs, and [SearchTeamMembers](api-endpoint:Team-SearchTeamMembers)
            to get team member IDs and current job assignments. Updates made to `published_shift_details`
            are ignored.

            If provided, the `start_at` and `end_at` timestamps must be in the time zone + offset of the
            shift location specified in `location_id`. Example for Pacific Standard Time: 2024-10-31T12:30:00-08:00

            To enable [optimistic concurrency](https://developer.squareup.com/docs/build-basics/common-api-patterns/optimistic-concurrency)
            control for the request, provide the current version of the shift in the `version` field.
            If the provided version doesn't match the server version, the request fails. If `version` is
            omitted, Square executes a blind write, potentially overwriting data from another publish request.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[UpdateScheduledShiftResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/labor/scheduled-shifts/{jsonable_encoder(id)}",
            method="PUT",
            json={
                "scheduled_shift": convert_and_respect_annotation_metadata(
                    object_=scheduled_shift, annotation=ScheduledShiftParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateScheduledShiftResponse,
                    construct_type(
                        type_=UpdateScheduledShiftResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def publish_scheduled_shift(
        self,
        id: str,
        *,
        idempotency_key: str,
        version: typing.Optional[int] = OMIT,
        scheduled_shift_notification_audience: typing.Optional[ScheduledShiftNotificationAudience] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[PublishScheduledShiftResponse]:
        """
        Publishes a scheduled shift. When a scheduled shift is published, Square keeps the
        `draft_shift_details` field as is and copies it to the `published_shift_details` field.

        Parameters
        ----------
        id : str
            The ID of the scheduled shift to publish.

        idempotency_key : str
            A unique identifier for the `PublishScheduledShift` request, used to ensure the
            [idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency)
            of the operation.

        version : typing.Optional[int]
            The current version of the scheduled shift, used to enable [optimistic concurrency](https://developer.squareup.com/docs/build-basics/common-api-patterns/optimistic-concurrency)
            control. If the provided version doesn't match the server version, the request fails.
            If omitted, Square executes a blind write, potentially overwriting data from another publish request.

        scheduled_shift_notification_audience : typing.Optional[ScheduledShiftNotificationAudience]
            Indicates whether Square should send an email notification to team members and
            which team members should receive the notification. The default value is `AFFECTED`.
            See [ScheduledShiftNotificationAudience](#type-scheduledshiftnotificationaudience) for possible values

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PublishScheduledShiftResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/labor/scheduled-shifts/{jsonable_encoder(id)}/publish",
            method="POST",
            json={
                "idempotency_key": idempotency_key,
                "version": version,
                "scheduled_shift_notification_audience": scheduled_shift_notification_audience,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PublishScheduledShiftResponse,
                    construct_type(
                        type_=PublishScheduledShiftResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create_timecard(
        self,
        *,
        timecard: TimecardParams,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CreateTimecardResponse]:
        """
        Creates a new `Timecard`.

        A `Timecard` represents a complete workday for a single team member.
        You must provide the following values in your request to this
        endpoint:

        - `location_id`
        - `team_member_id`
        - `start_at`

        An attempt to create a new `Timecard` can result in a `BAD_REQUEST` error when:
        - The `status` of the new `Timecard` is `OPEN` and the team member has another
        timecard with an `OPEN` status.
        - The `start_at` date is in the future.
        - The `start_at` or `end_at` date overlaps another timecard for the same team member.
        - The `Break` instances are set in the request and a break `start_at`
        is before the `Timecard.start_at`, a break `end_at` is after
        the `Timecard.end_at`, or both.

        Parameters
        ----------
        timecard : TimecardParams
            The `Timecard` to be created.

        idempotency_key : typing.Optional[str]
            A unique string value to ensure the idempotency of the operation.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CreateTimecardResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/labor/timecards",
            method="POST",
            json={
                "idempotency_key": idempotency_key,
                "timecard": convert_and_respect_annotation_metadata(
                    object_=timecard, annotation=TimecardParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateTimecardResponse,
                    construct_type(
                        type_=CreateTimecardResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def search_timecards(
        self,
        *,
        query: typing.Optional[TimecardQueryParams] = OMIT,
        limit: typing.Optional[int] = OMIT,
        cursor: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[SearchTimecardsResponse]:
        """
        Returns a paginated list of `Timecard` records for a business.
        The list to be returned can be filtered by:
        - Location IDs
        - Team member IDs
        - Timecard status (`OPEN` or `CLOSED`)
        - Timecard start
        - Timecard end
        - Workday details

        The list can be sorted by:
        - `START_AT`
        - `END_AT`
        - `CREATED_AT`
        - `UPDATED_AT`

        Parameters
        ----------
        query : typing.Optional[TimecardQueryParams]
            Query filters.

        limit : typing.Optional[int]
            The number of resources in a page (200 by default).

        cursor : typing.Optional[str]
            An opaque cursor for fetching the next page.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[SearchTimecardsResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/labor/timecards/search",
            method="POST",
            json={
                "query": convert_and_respect_annotation_metadata(
                    object_=query, annotation=TimecardQueryParams, direction="write"
                ),
                "limit": limit,
                "cursor": cursor,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SearchTimecardsResponse,
                    construct_type(
                        type_=SearchTimecardsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def retrieve_timecard(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[RetrieveTimecardResponse]:
        """
        Returns a single `Timecard` specified by `id`.

        Parameters
        ----------
        id : str
            The UUID for the `Timecard` being retrieved.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[RetrieveTimecardResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/labor/timecards/{jsonable_encoder(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    RetrieveTimecardResponse,
                    construct_type(
                        type_=RetrieveTimecardResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update_timecard(
        self, id: str, *, timecard: TimecardParams, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[UpdateTimecardResponse]:
        """
        Updates an existing `Timecard`.

        When adding a `Break` to a `Timecard`, any earlier `Break` instances in the `Timecard` have
        the `end_at` property set to a valid RFC-3339 datetime string.

        When closing a `Timecard`, all `Break` instances in the `Timecard` must be complete with `end_at`
        set on each `Break`.

        Parameters
        ----------
        id : str
            The ID of the object being updated.

        timecard : TimecardParams
            The updated `Timecard` object.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[UpdateTimecardResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/labor/timecards/{jsonable_encoder(id)}",
            method="PUT",
            json={
                "timecard": convert_and_respect_annotation_metadata(
                    object_=timecard, annotation=TimecardParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateTimecardResponse,
                    construct_type(
                        type_=UpdateTimecardResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete_timecard(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[DeleteTimecardResponse]:
        """
        Deletes a `Timecard`.

        Parameters
        ----------
        id : str
            The UUID for the `Timecard` being deleted.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[DeleteTimecardResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/labor/timecards/{jsonable_encoder(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DeleteTimecardResponse,
                    construct_type(
                        type_=DeleteTimecardResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncRawLaborClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create_scheduled_shift(
        self,
        *,
        scheduled_shift: ScheduledShiftParams,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CreateScheduledShiftResponse]:
        """
        Creates a scheduled shift by providing draft shift details such as job ID,
        team member assignment, and start and end times.

        The following `draft_shift_details` fields are required:
        - `location_id`
        - `job_id`
        - `start_at`
        - `end_at`

        Parameters
        ----------
        scheduled_shift : ScheduledShiftParams
            The scheduled shift with `draft_shift_details`.
            If needed, call [ListLocations](api-endpoint:Locations-ListLocations) to get location IDs,
            [ListJobs](api-endpoint:Team-ListJobs) to get job IDs, and [SearchTeamMembers](api-endpoint:Team-SearchTeamMembers)
            to get team member IDs and current job assignments.

            The `start_at` and `end_at` timestamps must be provided in the time zone + offset of the
            shift location specified in `location_id`. Example for Pacific Standard Time: 2024-10-31T12:30:00-08:00

        idempotency_key : typing.Optional[str]
            A unique identifier for the `CreateScheduledShift` request, used to ensure the
            [idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency)
            of the operation.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CreateScheduledShiftResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/labor/scheduled-shifts",
            method="POST",
            json={
                "idempotency_key": idempotency_key,
                "scheduled_shift": convert_and_respect_annotation_metadata(
                    object_=scheduled_shift, annotation=ScheduledShiftParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateScheduledShiftResponse,
                    construct_type(
                        type_=CreateScheduledShiftResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def bulk_publish_scheduled_shifts(
        self,
        *,
        scheduled_shifts: typing.Dict[str, BulkPublishScheduledShiftsDataParams],
        scheduled_shift_notification_audience: typing.Optional[ScheduledShiftNotificationAudience] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[BulkPublishScheduledShiftsResponse]:
        """
        Publishes 1 - 100 scheduled shifts. This endpoint takes a map of individual publish
        requests and returns a map of responses. When a scheduled shift is published, Square keeps
        the `draft_shift_details` field as is and copies it to the `published_shift_details` field.

        The minimum `start_at` and maximum `end_at` timestamps of all shifts in a
        `BulkPublishScheduledShifts` request must fall within a two-week period.

        Parameters
        ----------
        scheduled_shifts : typing.Dict[str, BulkPublishScheduledShiftsDataParams]
            A map of 1 to 100 key-value pairs that represent individual publish requests.

            - Each key is the ID of a scheduled shift you want to publish.
            - Each value is a `BulkPublishScheduledShiftsData` object that contains the
            `version` field or is an empty object.

        scheduled_shift_notification_audience : typing.Optional[ScheduledShiftNotificationAudience]
            Indicates whether Square should send email notifications to team members and
            which team members should receive the notifications. This setting applies to all shifts
            specified in the bulk operation. The default value is `AFFECTED`.
            See [ScheduledShiftNotificationAudience](#type-scheduledshiftnotificationaudience) for possible values

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[BulkPublishScheduledShiftsResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/labor/scheduled-shifts/bulk-publish",
            method="POST",
            json={
                "scheduled_shifts": convert_and_respect_annotation_metadata(
                    object_=scheduled_shifts,
                    annotation=typing.Dict[str, BulkPublishScheduledShiftsDataParams],
                    direction="write",
                ),
                "scheduled_shift_notification_audience": scheduled_shift_notification_audience,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    BulkPublishScheduledShiftsResponse,
                    construct_type(
                        type_=BulkPublishScheduledShiftsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def search_scheduled_shifts(
        self,
        *,
        query: typing.Optional[ScheduledShiftQueryParams] = OMIT,
        limit: typing.Optional[int] = OMIT,
        cursor: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[SearchScheduledShiftsResponse]:
        """
        Returns a paginated list of scheduled shifts, with optional filter and sort settings.
        By default, results are sorted by `start_at` in ascending order.

        Parameters
        ----------
        query : typing.Optional[ScheduledShiftQueryParams]
            Query conditions used to filter and sort the results.

        limit : typing.Optional[int]
            The maximum number of results to return in a single response page. The default value is 50.

        cursor : typing.Optional[str]
            The pagination cursor returned by the previous call to this endpoint. Provide
            this cursor to retrieve the next page of results for your original request. For more
            information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[SearchScheduledShiftsResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/labor/scheduled-shifts/search",
            method="POST",
            json={
                "query": convert_and_respect_annotation_metadata(
                    object_=query, annotation=ScheduledShiftQueryParams, direction="write"
                ),
                "limit": limit,
                "cursor": cursor,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SearchScheduledShiftsResponse,
                    construct_type(
                        type_=SearchScheduledShiftsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def retrieve_scheduled_shift(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[RetrieveScheduledShiftResponse]:
        """
        Retrieves a scheduled shift by ID.

        Parameters
        ----------
        id : str
            The ID of the scheduled shift to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[RetrieveScheduledShiftResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/labor/scheduled-shifts/{jsonable_encoder(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    RetrieveScheduledShiftResponse,
                    construct_type(
                        type_=RetrieveScheduledShiftResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update_scheduled_shift(
        self, id: str, *, scheduled_shift: ScheduledShiftParams, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[UpdateScheduledShiftResponse]:
        """
        Updates the draft shift details for a scheduled shift. This endpoint supports
        sparse updates, so only new, changed, or removed fields are required in the request.
        You must publish the shift to make updates public.

        You can make the following updates to `draft_shift_details`:
        - Change the `location_id`, `job_id`, `start_at`, and `end_at` fields.
        - Add, change, or clear the `team_member_id` and `notes` fields. To clear these fields,
        set the value to null.
        - Change the `is_deleted` field. To delete a scheduled shift, set `is_deleted` to true
        and then publish the shift.

        Parameters
        ----------
        id : str
            The ID of the scheduled shift to update.

        scheduled_shift : ScheduledShiftParams
            The scheduled shift with any updates in the `draft_shift_details` field.
            If needed, call [ListLocations](api-endpoint:Locations-ListLocations) to get location IDs,
            [ListJobs](api-endpoint:Team-ListJobs) to get job IDs, and [SearchTeamMembers](api-endpoint:Team-SearchTeamMembers)
            to get team member IDs and current job assignments. Updates made to `published_shift_details`
            are ignored.

            If provided, the `start_at` and `end_at` timestamps must be in the time zone + offset of the
            shift location specified in `location_id`. Example for Pacific Standard Time: 2024-10-31T12:30:00-08:00

            To enable [optimistic concurrency](https://developer.squareup.com/docs/build-basics/common-api-patterns/optimistic-concurrency)
            control for the request, provide the current version of the shift in the `version` field.
            If the provided version doesn't match the server version, the request fails. If `version` is
            omitted, Square executes a blind write, potentially overwriting data from another publish request.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[UpdateScheduledShiftResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/labor/scheduled-shifts/{jsonable_encoder(id)}",
            method="PUT",
            json={
                "scheduled_shift": convert_and_respect_annotation_metadata(
                    object_=scheduled_shift, annotation=ScheduledShiftParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateScheduledShiftResponse,
                    construct_type(
                        type_=UpdateScheduledShiftResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def publish_scheduled_shift(
        self,
        id: str,
        *,
        idempotency_key: str,
        version: typing.Optional[int] = OMIT,
        scheduled_shift_notification_audience: typing.Optional[ScheduledShiftNotificationAudience] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[PublishScheduledShiftResponse]:
        """
        Publishes a scheduled shift. When a scheduled shift is published, Square keeps the
        `draft_shift_details` field as is and copies it to the `published_shift_details` field.

        Parameters
        ----------
        id : str
            The ID of the scheduled shift to publish.

        idempotency_key : str
            A unique identifier for the `PublishScheduledShift` request, used to ensure the
            [idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency)
            of the operation.

        version : typing.Optional[int]
            The current version of the scheduled shift, used to enable [optimistic concurrency](https://developer.squareup.com/docs/build-basics/common-api-patterns/optimistic-concurrency)
            control. If the provided version doesn't match the server version, the request fails.
            If omitted, Square executes a blind write, potentially overwriting data from another publish request.

        scheduled_shift_notification_audience : typing.Optional[ScheduledShiftNotificationAudience]
            Indicates whether Square should send an email notification to team members and
            which team members should receive the notification. The default value is `AFFECTED`.
            See [ScheduledShiftNotificationAudience](#type-scheduledshiftnotificationaudience) for possible values

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PublishScheduledShiftResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/labor/scheduled-shifts/{jsonable_encoder(id)}/publish",
            method="POST",
            json={
                "idempotency_key": idempotency_key,
                "version": version,
                "scheduled_shift_notification_audience": scheduled_shift_notification_audience,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PublishScheduledShiftResponse,
                    construct_type(
                        type_=PublishScheduledShiftResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create_timecard(
        self,
        *,
        timecard: TimecardParams,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CreateTimecardResponse]:
        """
        Creates a new `Timecard`.

        A `Timecard` represents a complete workday for a single team member.
        You must provide the following values in your request to this
        endpoint:

        - `location_id`
        - `team_member_id`
        - `start_at`

        An attempt to create a new `Timecard` can result in a `BAD_REQUEST` error when:
        - The `status` of the new `Timecard` is `OPEN` and the team member has another
        timecard with an `OPEN` status.
        - The `start_at` date is in the future.
        - The `start_at` or `end_at` date overlaps another timecard for the same team member.
        - The `Break` instances are set in the request and a break `start_at`
        is before the `Timecard.start_at`, a break `end_at` is after
        the `Timecard.end_at`, or both.

        Parameters
        ----------
        timecard : TimecardParams
            The `Timecard` to be created.

        idempotency_key : typing.Optional[str]
            A unique string value to ensure the idempotency of the operation.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CreateTimecardResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/labor/timecards",
            method="POST",
            json={
                "idempotency_key": idempotency_key,
                "timecard": convert_and_respect_annotation_metadata(
                    object_=timecard, annotation=TimecardParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateTimecardResponse,
                    construct_type(
                        type_=CreateTimecardResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def search_timecards(
        self,
        *,
        query: typing.Optional[TimecardQueryParams] = OMIT,
        limit: typing.Optional[int] = OMIT,
        cursor: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[SearchTimecardsResponse]:
        """
        Returns a paginated list of `Timecard` records for a business.
        The list to be returned can be filtered by:
        - Location IDs
        - Team member IDs
        - Timecard status (`OPEN` or `CLOSED`)
        - Timecard start
        - Timecard end
        - Workday details

        The list can be sorted by:
        - `START_AT`
        - `END_AT`
        - `CREATED_AT`
        - `UPDATED_AT`

        Parameters
        ----------
        query : typing.Optional[TimecardQueryParams]
            Query filters.

        limit : typing.Optional[int]
            The number of resources in a page (200 by default).

        cursor : typing.Optional[str]
            An opaque cursor for fetching the next page.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[SearchTimecardsResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/labor/timecards/search",
            method="POST",
            json={
                "query": convert_and_respect_annotation_metadata(
                    object_=query, annotation=TimecardQueryParams, direction="write"
                ),
                "limit": limit,
                "cursor": cursor,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SearchTimecardsResponse,
                    construct_type(
                        type_=SearchTimecardsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def retrieve_timecard(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[RetrieveTimecardResponse]:
        """
        Returns a single `Timecard` specified by `id`.

        Parameters
        ----------
        id : str
            The UUID for the `Timecard` being retrieved.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[RetrieveTimecardResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/labor/timecards/{jsonable_encoder(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    RetrieveTimecardResponse,
                    construct_type(
                        type_=RetrieveTimecardResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update_timecard(
        self, id: str, *, timecard: TimecardParams, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[UpdateTimecardResponse]:
        """
        Updates an existing `Timecard`.

        When adding a `Break` to a `Timecard`, any earlier `Break` instances in the `Timecard` have
        the `end_at` property set to a valid RFC-3339 datetime string.

        When closing a `Timecard`, all `Break` instances in the `Timecard` must be complete with `end_at`
        set on each `Break`.

        Parameters
        ----------
        id : str
            The ID of the object being updated.

        timecard : TimecardParams
            The updated `Timecard` object.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[UpdateTimecardResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/labor/timecards/{jsonable_encoder(id)}",
            method="PUT",
            json={
                "timecard": convert_and_respect_annotation_metadata(
                    object_=timecard, annotation=TimecardParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateTimecardResponse,
                    construct_type(
                        type_=UpdateTimecardResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete_timecard(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[DeleteTimecardResponse]:
        """
        Deletes a `Timecard`.

        Parameters
        ----------
        id : str
            The UUID for the `Timecard` being deleted.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[DeleteTimecardResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/labor/timecards/{jsonable_encoder(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DeleteTimecardResponse,
                    construct_type(
                        type_=DeleteTimecardResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .address import Address
from .invoice_recipient_tax_ids import InvoiceRecipientTaxIds
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class InvoiceRecipient(UncheckedBaseModel):
    """
    Represents a snapshot of customer data. This object stores customer data that is displayed on the invoice
    and that Square uses to deliver the invoice.

    When you provide a customer ID for a draft invoice, Square retrieves the associated customer profile and populates
    the remaining `InvoiceRecipient` fields. You cannot update these fields after the invoice is published.
    Square updates the customer ID in response to a merge operation, but does not update other fields.
    """

    customer_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the customer. This is the customer profile ID that 
    you provide when creating a draft invoice.
    """

    given_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The recipient's given (that is, first) name.
    """

    family_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The recipient's family (that is, last) name.
    """

    email_address: typing.Optional[str] = pydantic.Field(default=None)
    """
    The recipient's email address.
    """

    address: typing.Optional[Address] = pydantic.Field(default=None)
    """
    The recipient's physical address.
    """

    phone_number: typing.Optional[str] = pydantic.Field(default=None)
    """
    The recipient's phone number.
    """

    company_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the recipient's company.
    """

    tax_ids: typing.Optional[InvoiceRecipientTaxIds] = pydantic.Field(default=None)
    """
    The recipient's tax IDs. The country of the seller account determines whether this field 
    is available for the customer. For more information, see [Invoice recipient tax IDs](https://developer.squareup.com/docs/invoices-api/overview#recipient-tax-ids).
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.client_wrapper import SyncClientWrapper
from .raw_client import RawDevicesClient
from .codes.client import CodesClient
import typing
from ..types.sort_order import SortOrder
from ..core.request_options import RequestOptions
from ..core.pagination import SyncPager
from ..types.device import Device
from ..types.list_devices_response import ListDevicesResponse
from ..core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..types.get_device_response import GetDeviceResponse
from ..core.client_wrapper import AsyncClientWrapper
from .raw_client import AsyncRawDevicesClient
from .codes.client import AsyncCodesClient
from ..core.pagination import AsyncPager


class DevicesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawDevicesClient(client_wrapper=client_wrapper)
        self.codes = CodesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawDevicesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawDevicesClient
        """
        return self._raw_client

    def list(
        self,
        *,
        cursor: typing.Optional[str] = None,
        sort_order: typing.Optional[SortOrder] = None,
        limit: typing.Optional[int] = None,
        location_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[Device]:
        """
        List devices associated with the merchant. Currently, only Terminal API
        devices are supported.

        Parameters
        ----------
        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this cursor to retrieve the next set of results for the original query.
            See [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination) for more information.

        sort_order : typing.Optional[SortOrder]
            The order in which results are listed.
            - `ASC` - Oldest to newest.
            - `DESC` - Newest to oldest (default).

        limit : typing.Optional[int]
            The number of results to return in a single page.

        location_id : typing.Optional[str]
            If present, only returns devices at the target location.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[Device]
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        response = client.devices.list()
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        _response = self._raw_client._client_wrapper.httpx_client.request(
            "v2/devices",
            method="GET",
            params={
                "cursor": cursor,
                "sort_order": sort_order,
                "limit": limit,
                "location_id": location_id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListDevicesResponse,
                    construct_type(
                        type_=ListDevicesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list(
                    cursor=_parsed_next,
                    sort_order=sort_order,
                    limit=limit,
                    location_id=location_id,
                    request_options=request_options,
                )
                _items = _parsed_response.devices
                return SyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(self, device_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> GetDeviceResponse:
        """
        Retrieves Device with the associated `device_id`.

        Parameters
        ----------
        device_id : str
            The unique ID for the desired `Device`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetDeviceResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.devices.get(
            device_id="device_id",
        )
        """
        response = self._raw_client.get(device_id, request_options=request_options)
        return response.data


class AsyncDevicesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawDevicesClient(client_wrapper=client_wrapper)
        self.codes = AsyncCodesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawDevicesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawDevicesClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        cursor: typing.Optional[str] = None,
        sort_order: typing.Optional[SortOrder] = None,
        limit: typing.Optional[int] = None,
        location_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[Device]:
        """
        List devices associated with the merchant. Currently, only Terminal API
        devices are supported.

        Parameters
        ----------
        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this cursor to retrieve the next set of results for the original query.
            See [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination) for more information.

        sort_order : typing.Optional[SortOrder]
            The order in which results are listed.
            - `ASC` - Oldest to newest.
            - `DESC` - Newest to oldest (default).

        limit : typing.Optional[int]
            The number of results to return in a single page.

        location_id : typing.Optional[str]
            If present, only returns devices at the target location.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[Device]
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.devices.list()
            async for item in response:
                yield item
            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        _response = await self._raw_client._client_wrapper.httpx_client.request(
            "v2/devices",
            method="GET",
            params={
                "cursor": cursor,
                "sort_order": sort_order,
                "limit": limit,
                "location_id": location_id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListDevicesResponse,
                    construct_type(
                        type_=ListDevicesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list(
                    cursor=_parsed_next,
                    sort_order=sort_order,
                    limit=limit,
                    location_id=location_id,
                    request_options=request_options,
                )
                _items = _parsed_response.devices
                return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(
        self, device_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetDeviceResponse:
        """
        Retrieves Device with the associated `device_id`.

        Parameters
        ----------
        device_id : str
            The unique ID for the desired `Device`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetDeviceResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.devices.get(
                device_id="device_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.get(device_id, request_options=request_options)
        return response.data



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .error import Error
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class ObtainTokenResponse(UncheckedBaseModel):
    """
    Represents an [ObtainToken](api-endpoint:OAuth-ObtainToken) response.
    """

    access_token: typing.Optional[str] = pydantic.Field(default=None)
    """
    An OAuth access token used to authorize Square API requests on behalf of the seller.
    Include this token as a bearer token in the `Authorization` header of your API requests.
    
    OAuth access tokens expire in 30 days (except `short_lived` access tokens). You should call
    `ObtainToken` and provide the returned `refresh_token` to get a new access token well before
    the current one expires. For more information, see [OAuth API: Walkthrough](https://developer.squareup.com/docs/oauth-api/walkthrough).
    """

    token_type: typing.Optional[str] = pydantic.Field(default=None)
    """
    The type of access token. This value is always `bearer`.
    """

    expires_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The timestamp of when the `access_token` expires, in [ISO 8601](http://www.iso.org/iso/home/standards/iso8601.htm) format.
    """

    merchant_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the authorizing [merchant](entity:Merchant) (seller), which represents a business.
    """

    subscription_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    __LEGACY__ The ID of merchant's subscription.
    The ID is only present if the merchant signed up for a subscription plan during authorization.
    """

    plan_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    __LEGACY__ The ID of the subscription plan the merchant signed
    up for. The ID is only present if the merchant signed up for a subscription plan during
    authorization.
    """

    id_token: typing.Optional[str] = pydantic.Field(default=None)
    """
    The OpenID token that belongs to this person. This token is only present if the
    `OPENID` scope is included in the authorization request.
    
    Deprecated at version 2021-09-15. Square doesn't support OpenID or other single sign-on (SSO)
    protocols on top of OAuth.
    """

    refresh_token: typing.Optional[str] = pydantic.Field(default=None)
    """
    A refresh token that can be used in an `ObtainToken` request to generate a new access token.
    
    With the code flow:
    - For the `authorization_code` grant type, the refresh token is multi-use and never expires.
    - For the `refresh_token` grant type, the response returns the same refresh token.
    
    With the PKCE flow:
    - For the `authorization_code` grant type, the refresh token is single-use and expires in 90 days.
    - For the `refresh_token` grant type, the refresh token is a new single-use refresh token that expires in 90 days.
    
    For more information, see [Refresh, Revoke, and Limit the Scope of OAuth Tokens](https://developer.squareup.com/docs/oauth-api/refresh-revoke-limit-scope).
    """

    short_lived: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Indicates whether the access_token is short lived. If `true`, the access token expires
    in 24 hours. If `false`, the access token expires in 30 days.
    """

    errors: typing.Optional[typing.List[Error]] = pydantic.Field(default=None)
    """
    Any errors that occurred during the request.
    """

    refresh_token_expires_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The timestamp of when the `refresh_token` expires, in [ISO 8601](http://www.iso.org/iso/home/standards/iso8601.htm)
    format.
    
    This field is only returned for the PKCE flow.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .error import Error
import pydantic
from .device_code import DeviceCode
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class ListDeviceCodesResponse(UncheckedBaseModel):
    errors: typing.Optional[typing.List[Error]] = pydantic.Field(default=None)
    """
    Any errors that occurred during the request.
    """

    device_codes: typing.Optional[typing.List[DeviceCode]] = pydantic.Field(default=None)
    """
    The queried DeviceCode.
    """

    cursor: typing.Optional[str] = pydantic.Field(default=None)
    """
    A pagination cursor to retrieve the next set of results for your
    original query to the endpoint. This value is present only if the request
    succeeded and additional results are available.
    
    See [Paginating results](https://developer.squareup.com/docs/working-with-apis/pagination) for more information.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .cash_drawer_event_type import CashDrawerEventType
from .money import Money
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class CashDrawerShiftEvent(UncheckedBaseModel):
    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique ID of the event.
    """

    event_type: typing.Optional[CashDrawerEventType] = pydantic.Field(default=None)
    """
    The type of cash drawer shift event.
    See [CashDrawerEventType](#type-cashdrawereventtype) for possible values
    """

    event_money: typing.Optional[Money] = pydantic.Field(default=None)
    """
    The amount of money that was added to or removed from the cash drawer
    in the event. The amount can be positive (for added money)
    or zero (for other tender type payments). The addition or removal of money can be determined by
    by the event type.
    """

    created_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The event time in RFC 3339 format.
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    An optional description of the event, entered by the employee that
    created the event.
    """

    team_member_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the team member that created the event.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .time_range import TimeRangeParams


class SearchEventsFilterParams(typing_extensions.TypedDict):
    """
    Criteria to filter events by.
    """

    event_types: typing_extensions.NotRequired[typing.Optional[typing.Sequence[str]]]
    """
    Filter events by event types.
    """

    merchant_ids: typing_extensions.NotRequired[typing.Optional[typing.Sequence[str]]]
    """
    Filter events by merchant.
    """

    location_ids: typing_extensions.NotRequired[typing.Optional[typing.Sequence[str]]]
    """
    Filter events by location.
    """

    created_at: typing_extensions.NotRequired[TimeRangeParams]
    """
    Filter events by when they were created.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .error import Error
import pydantic
from .transaction import Transaction
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class ListTransactionsResponse(UncheckedBaseModel):
    """
    Defines the fields that are included in the response body of
    a request to the [ListTransactions](api-endpoint:Transactions-ListTransactions) endpoint.

    One of `errors` or `transactions` is present in a given response (never both).
    """

    errors: typing.Optional[typing.List[Error]] = pydantic.Field(default=None)
    """
    Any errors that occurred during the request.
    """

    transactions: typing.Optional[typing.List[Transaction]] = pydantic.Field(default=None)
    """
    An array of transactions that match your query.
    """

    cursor: typing.Optional[str] = pydantic.Field(default=None)
    """
    A pagination cursor for retrieving the next set of results,
    if any remain. Provide this value as the `cursor` parameter in a subsequent
    request to this endpoint.
    
    See [Paginating results](https://developer.squareup.com/docs/working-with-apis/pagination) for more information.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from . import cards, custom_attribute_definitions, custom_attributes, groups, segments

__all__ = ["cards", "custom_attribute_definitions", "custom_attributes", "groups", "segments"]



# This file was auto-generated by Fern from our API Definition.

import typing

CheckoutLocationSettingsBrandingHeaderType = typing.Union[
    typing.Literal["BUSINESS_NAME", "FRAMED_LOGO", "FULL_WIDTH_LOGO"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .dispute import DisputeParams


class SubmitEvidenceResponseParams(typing_extensions.TypedDict):
    """
    Defines the fields in a `SubmitEvidence` response.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Information about errors encountered during the request.
    """

    dispute: typing_extensions.NotRequired[DisputeParams]
    """
    The `Dispute` for which evidence was submitted.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing


class LocationBookingProfileParams(typing_extensions.TypedDict):
    """
    The booking profile of a seller's location, including the location's ID and whether the location is enabled for online booking.
    """

    location_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The ID of the [location](entity:Location).
    """

    booking_site_url: typing_extensions.NotRequired[typing.Optional[str]]
    """
    Url for the online booking site for this location.
    """

    online_booking_enabled: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    Indicates whether the location is enabled for online booking.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions


class CreateInvoiceAttachmentRequestDataParams(typing_extensions.TypedDict):
    """
    Represents a [CreateInvoiceAttachment](api-endpoint:Invoices-CreateInvoiceAttachment) request.
    """

    idempotency_key: typing_extensions.NotRequired[str]
    """
    A unique string that identifies the `CreateInvoiceAttachment` request.
    For more information, see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).
    """

    description: typing_extensions.NotRequired[str]
    """
    The description of the attachment to display on the invoice.
    """



# This file was auto-generated by Fern from our API Definition.

import typing

DisputeState = typing.Union[
    typing.Literal[
        "INQUIRY_EVIDENCE_REQUIRED",
        "INQUIRY_PROCESSING",
        "INQUIRY_CLOSED",
        "EVIDENCE_REQUIRED",
        "PROCESSING",
        "WON",
        "LOST",
        "ACCEPTED",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from ..requests.gift_card import GiftCardParams
from ..core.request_options import RequestOptions
from ..core.http_response import HttpResponse
from ..types.create_gift_card_response import CreateGiftCardResponse
from ..core.serialization import convert_and_respect_annotation_metadata
from ..core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..types.get_gift_card_from_gan_response import GetGiftCardFromGanResponse
from ..types.get_gift_card_from_nonce_response import GetGiftCardFromNonceResponse
from ..types.link_customer_to_gift_card_response import LinkCustomerToGiftCardResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..types.unlink_customer_from_gift_card_response import UnlinkCustomerFromGiftCardResponse
from ..types.get_gift_card_response import GetGiftCardResponse
from ..core.client_wrapper import AsyncClientWrapper
from ..core.http_response import AsyncHttpResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawGiftCardsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create(
        self,
        *,
        idempotency_key: str,
        location_id: str,
        gift_card: GiftCardParams,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CreateGiftCardResponse]:
        """
        Creates a digital gift card or registers a physical (plastic) gift card. The resulting gift card
        has a `PENDING` state. To activate a gift card so that it can be redeemed for purchases, call
        [CreateGiftCardActivity](api-endpoint:GiftCardActivities-CreateGiftCardActivity) and create an `ACTIVATE`
        activity with the initial balance. Alternatively, you can use [RefundPayment](api-endpoint:Refunds-RefundPayment)
        to refund a payment to the new gift card.

        Parameters
        ----------
        idempotency_key : str
            A unique identifier for this request, used to ensure idempotency. For more information,
            see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

        location_id : str
            The ID of the [location](entity:Location) where the gift card should be registered for
            reporting purposes. Gift cards can be redeemed at any of the seller's locations.

        gift_card : GiftCardParams
            The gift card to create. The `type` field is required for this request. The `gan_source`
            and `gan` fields are included as follows:

            To direct Square to generate a 16-digit GAN, omit `gan_source` and `gan`.

            To provide a custom GAN, include `gan_source` and `gan`.
            - For `gan_source`, specify `OTHER`.
            - For `gan`, provide a custom GAN containing 8 to 20 alphanumeric characters. The GAN must be
            unique for the seller and cannot start with the same bank identification number (BIN) as major
            credit cards. Do not use GANs that are easy to guess (such as 12345678) because they greatly
            increase the risk of fraud. It is the responsibility of the developer to ensure the security
            of their custom GANs. For more information, see
            [Custom GANs](https://developer.squareup.com/docs/gift-cards/using-gift-cards-api#custom-gans).

            To register an unused, physical gift card that the seller previously ordered from Square,
            include `gan` and provide the GAN that is printed on the gift card.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CreateGiftCardResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/gift-cards",
            method="POST",
            json={
                "idempotency_key": idempotency_key,
                "location_id": location_id,
                "gift_card": convert_and_respect_annotation_metadata(
                    object_=gift_card, annotation=GiftCardParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateGiftCardResponse,
                    construct_type(
                        type_=CreateGiftCardResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_from_gan(
        self, *, gan: str, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetGiftCardFromGanResponse]:
        """
        Retrieves a gift card using the gift card account number (GAN).

        Parameters
        ----------
        gan : str
            The gift card account number (GAN) of the gift card to retrieve.
            The maximum length of a GAN is 255 digits to account for third-party GANs that have been imported.
            Square-issued gift cards have 16-digit GANs.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetGiftCardFromGanResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/gift-cards/from-gan",
            method="POST",
            json={
                "gan": gan,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetGiftCardFromGanResponse,
                    construct_type(
                        type_=GetGiftCardFromGanResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_from_nonce(
        self, *, nonce: str, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetGiftCardFromNonceResponse]:
        """
        Retrieves a gift card using a secure payment token that represents the gift card.

        Parameters
        ----------
        nonce : str
            The payment token of the gift card to retrieve. Payment tokens are generated by the
            Web Payments SDK or In-App Payments SDK.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetGiftCardFromNonceResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/gift-cards/from-nonce",
            method="POST",
            json={
                "nonce": nonce,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetGiftCardFromNonceResponse,
                    construct_type(
                        type_=GetGiftCardFromNonceResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def link_customer(
        self, gift_card_id: str, *, customer_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[LinkCustomerToGiftCardResponse]:
        """
        Links a customer to a gift card, which is also referred to as adding a card on file.

        Parameters
        ----------
        gift_card_id : str
            The ID of the gift card to be linked.

        customer_id : str
            The ID of the customer to link to the gift card.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[LinkCustomerToGiftCardResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/gift-cards/{jsonable_encoder(gift_card_id)}/link-customer",
            method="POST",
            json={
                "customer_id": customer_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    LinkCustomerToGiftCardResponse,
                    construct_type(
                        type_=LinkCustomerToGiftCardResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def unlink_customer(
        self, gift_card_id: str, *, customer_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[UnlinkCustomerFromGiftCardResponse]:
        """
        Unlinks a customer from a gift card, which is also referred to as removing a card on file.

        Parameters
        ----------
        gift_card_id : str
            The ID of the gift card to be unlinked.

        customer_id : str
            The ID of the customer to unlink from the gift card.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[UnlinkCustomerFromGiftCardResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/gift-cards/{jsonable_encoder(gift_card_id)}/unlink-customer",
            method="POST",
            json={
                "customer_id": customer_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UnlinkCustomerFromGiftCardResponse,
                    construct_type(
                        type_=UnlinkCustomerFromGiftCardResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetGiftCardResponse]:
        """
        Retrieves a gift card using the gift card ID.

        Parameters
        ----------
        id : str
            The ID of the gift card to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetGiftCardResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/gift-cards/{jsonable_encoder(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetGiftCardResponse,
                    construct_type(
                        type_=GetGiftCardResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncRawGiftCardsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create(
        self,
        *,
        idempotency_key: str,
        location_id: str,
        gift_card: GiftCardParams,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CreateGiftCardResponse]:
        """
        Creates a digital gift card or registers a physical (plastic) gift card. The resulting gift card
        has a `PENDING` state. To activate a gift card so that it can be redeemed for purchases, call
        [CreateGiftCardActivity](api-endpoint:GiftCardActivities-CreateGiftCardActivity) and create an `ACTIVATE`
        activity with the initial balance. Alternatively, you can use [RefundPayment](api-endpoint:Refunds-RefundPayment)
        to refund a payment to the new gift card.

        Parameters
        ----------
        idempotency_key : str
            A unique identifier for this request, used to ensure idempotency. For more information,
            see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

        location_id : str
            The ID of the [location](entity:Location) where the gift card should be registered for
            reporting purposes. Gift cards can be redeemed at any of the seller's locations.

        gift_card : GiftCardParams
            The gift card to create. The `type` field is required for this request. The `gan_source`
            and `gan` fields are included as follows:

            To direct Square to generate a 16-digit GAN, omit `gan_source` and `gan`.

            To provide a custom GAN, include `gan_source` and `gan`.
            - For `gan_source`, specify `OTHER`.
            - For `gan`, provide a custom GAN containing 8 to 20 alphanumeric characters. The GAN must be
            unique for the seller and cannot start with the same bank identification number (BIN) as major
            credit cards. Do not use GANs that are easy to guess (such as 12345678) because they greatly
            increase the risk of fraud. It is the responsibility of the developer to ensure the security
            of their custom GANs. For more information, see
            [Custom GANs](https://developer.squareup.com/docs/gift-cards/using-gift-cards-api#custom-gans).

            To register an unused, physical gift card that the seller previously ordered from Square,
            include `gan` and provide the GAN that is printed on the gift card.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CreateGiftCardResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/gift-cards",
            method="POST",
            json={
                "idempotency_key": idempotency_key,
                "location_id": location_id,
                "gift_card": convert_and_respect_annotation_metadata(
                    object_=gift_card, annotation=GiftCardParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateGiftCardResponse,
                    construct_type(
                        type_=CreateGiftCardResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_from_gan(
        self, *, gan: str, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetGiftCardFromGanResponse]:
        """
        Retrieves a gift card using the gift card account number (GAN).

        Parameters
        ----------
        gan : str
            The gift card account number (GAN) of the gift card to retrieve.
            The maximum length of a GAN is 255 digits to account for third-party GANs that have been imported.
            Square-issued gift cards have 16-digit GANs.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetGiftCardFromGanResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/gift-cards/from-gan",
            method="POST",
            json={
                "gan": gan,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetGiftCardFromGanResponse,
                    construct_type(
                        type_=GetGiftCardFromGanResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_from_nonce(
        self, *, nonce: str, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetGiftCardFromNonceResponse]:
        """
        Retrieves a gift card using a secure payment token that represents the gift card.

        Parameters
        ----------
        nonce : str
            The payment token of the gift card to retrieve. Payment tokens are generated by the
            Web Payments SDK or In-App Payments SDK.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetGiftCardFromNonceResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/gift-cards/from-nonce",
            method="POST",
            json={
                "nonce": nonce,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetGiftCardFromNonceResponse,
                    construct_type(
                        type_=GetGiftCardFromNonceResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def link_customer(
        self, gift_card_id: str, *, customer_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[LinkCustomerToGiftCardResponse]:
        """
        Links a customer to a gift card, which is also referred to as adding a card on file.

        Parameters
        ----------
        gift_card_id : str
            The ID of the gift card to be linked.

        customer_id : str
            The ID of the customer to link to the gift card.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[LinkCustomerToGiftCardResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/gift-cards/{jsonable_encoder(gift_card_id)}/link-customer",
            method="POST",
            json={
                "customer_id": customer_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    LinkCustomerToGiftCardResponse,
                    construct_type(
                        type_=LinkCustomerToGiftCardResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def unlink_customer(
        self, gift_card_id: str, *, customer_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[UnlinkCustomerFromGiftCardResponse]:
        """
        Unlinks a customer from a gift card, which is also referred to as removing a card on file.

        Parameters
        ----------
        gift_card_id : str
            The ID of the gift card to be unlinked.

        customer_id : str
            The ID of the customer to unlink from the gift card.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[UnlinkCustomerFromGiftCardResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/gift-cards/{jsonable_encoder(gift_card_id)}/unlink-customer",
            method="POST",
            json={
                "customer_id": customer_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UnlinkCustomerFromGiftCardResponse,
                    construct_type(
                        type_=UnlinkCustomerFromGiftCardResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetGiftCardResponse]:
        """
        Retrieves a gift card using the gift card ID.

        Parameters
        ----------
        id : str
            The ID of the gift card to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetGiftCardResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/gift-cards/{jsonable_encoder(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetGiftCardResponse,
                    construct_type(
                        type_=GetGiftCardResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .subscription_test_result import SubscriptionTestResultParams


class TestWebhookSubscriptionResponseParams(typing_extensions.TypedDict):
    """
    Defines the fields that are included in the response body of
    a request to the [TestWebhookSubscription](api-endpoint:WebhookSubscriptions-TestWebhookSubscription) endpoint.

    Note: If there are errors processing the request, the [SubscriptionTestResult](entity:SubscriptionTestResult) field is not
    present.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Information on errors encountered during the request.
    """

    subscription_test_result: typing_extensions.NotRequired[SubscriptionTestResultParams]
    """
    The [SubscriptionTestResult](entity:SubscriptionTestResult).
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions


class CustomerGroupParams(typing_extensions.TypedDict):
    """
    Represents a group of customer profiles.

    Customer groups can be created, be modified, and have their membership defined using
    the Customers API or within the Customer Directory in the Square Seller Dashboard or Point of Sale.
    """

    id: typing_extensions.NotRequired[str]
    """
    A unique Square-generated ID for the customer group.
    """

    name: str
    """
    The name of the customer group.
    """

    created_at: typing_extensions.NotRequired[str]
    """
    The timestamp when the customer group was created, in RFC 3339 format.
    """

    updated_at: typing_extensions.NotRequired[str]
    """
    The timestamp when the customer group was last updated, in RFC 3339 format.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing


class CatalogIdMappingParams(typing_extensions.TypedDict):
    """
    A mapping between a temporary client-supplied ID and a permanent server-generated ID.

    When calling [UpsertCatalogObject](api-endpoint:Catalog-UpsertCatalogObject) or
    [BatchUpsertCatalogObjects](api-endpoint:Catalog-BatchUpsertCatalogObjects) to
    create a [CatalogObject](entity:CatalogObject) instance, you can supply
    a temporary ID for the to-be-created object, especially when the object is to be referenced
    elsewhere in the same request body. This temporary ID can be any string unique within
    the call, but must be prefixed by "#".

    After the request is submitted and the object created, a permanent server-generated ID is assigned
    to the new object. The permanent ID is unique across the Square catalog.
    """

    client_object_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The client-supplied temporary `#`-prefixed ID for a new `CatalogObject`.
    """

    object_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The permanent ID for the CatalogObject created by the server.
    """
